/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Stores information about creature. Used for GA developing movement. 
  * @see WalkGA
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Creature
{
  
  /** First joint */
  Joint joint;
  
  
  // for drawing
  public int x[]=new int[WalkParams.MAX_POINTS];
  public int y[]=new int[WalkParams.MAX_POINTS];
  public int xyEnd=0;
  public int from[]=new int[WalkParams.MAX_POINTS];
  public int to[]=new int[WalkParams.MAX_POINTS];
  public int ftEnd=0;
  
  // for population
  public CreatureChromosome sample;
  public CreatureChromosome chr;
  public int position;
  public int positions;
  
  public Population population;
  
  public int lastX=0;
  public int lastId=0;
  
  public int ySize=300;
  public int xSize=300;
  
  public int shiftX=-50;
  public int lastShiftX=-50;
  
  public void dump()
  {
    System.out.println("Creature dump");
    System.out.println("xy "+xyEnd);
    for(int i=0; i<xyEnd; i++) System.out.print("["+x[i]+","+y[i]+"]");
    System.out.println();
    System.out.println("ft "+ftEnd);
    for(int i=0; i<ftEnd; i++) System.out.print("["+from[i]+","+to[i]+"]");
    System.out.println();
    System.out.println("=================");
  }
  
  /** Construct new empty creature. Use addJoint to construct useful creature.
    * @see #addJoint(int, Joint)
    */
  public Creature()
  {
    joint=new Joint(0,0,0);
  }
  
  /** Adds joint to joint with ID from parameter id (start with 0).
    * @see Joint
    */
  public boolean addJoint(int id, Joint j)
  {
    return joint.addJoint(id,j);
  }
  
  /** Creates new population for this creature with aPosition positions for movement. */
  public void createPopulation(int aPositions)
  {
    positions=aPositions;
    chr = sample = new CreatureChromosome(Joint.idCount, positions, this);
    population = new Population(sample, WalkParams.POP_SIZE);
  }
  
  int llastId;
  int llastX;
  
  /** Computes coordinates for drawing for specified position and chromosome. */
  public void computeDraw(int aPosition, int chromosomeIndex)
  {
    shiftX=lastShiftX;
    lastId=llastId;
    lastX=llastX;
    position=aPosition;
    chr=(CreatureChromosome)(population.pop[chromosomeIndex]);
    computeDraw();
    lastShiftX=shiftX;
    llastId=lastId;
    llastX=lastX;
  }
  
  /** Computes coordinates for drawing. */
  public void computeDraw()
  {
    xyEnd=0;
    ftEnd=0;
    x[0]=150;
    y[0]=50;
    joint.startComputeDraw(this,x[0],y[0],0);
    
    xyEnd++;
    ftEnd++;
    int miny=Integer.MAX_VALUE;
    int prevId=lastId;
    for(int i=0; i<xyEnd; i++)
      {
        if(miny>y[i]) 
          {
            miny=y[i];
            lastId=i;
          }
      }
    int sx=x[prevId]-lastX;
    lastX=x[lastId];
    shiftX+=sx;
    int maxx=Integer.MIN_VALUE;
    int minx=Integer.MAX_VALUE;
    for(int i=0; i<xyEnd; i++)
      {
        y[i]=ySize-(y[i]-miny);
        int mm=x[i]-=shiftX;
        if(maxx<mm) maxx=mm;
        if(minx>mm) minx=mm;
      }
    if(maxx>xSize) shiftX+=xSize;  
    if(minx<0) shiftX-=xSize;  
  }
  
  /** Evaluates fitness of creature chromosome. */
  public int evaluate(CreatureChromosome cc)
  {
    chr=cc;
    shiftX=-50;
    joint.resetVar();
    position=0;
    computeDraw();
    int XX=x[0];
    shiftX=-50;
    joint.resetVar();
    for(int repeat=0; repeat<4; repeat++) // dvakrat, jestli se muze vratit a pokracovat  
    for(int i=0; i<WalkParams.POSITIONS; i++)
      {
        position=i;
        computeDraw();
      }
    position=0;
    computeDraw();
    XX=x[0]-XX;
    //if(XX>xSize) XX+=xSize;  
    //if(XX<0) XX-=xSize;  
    shiftX=-50;
    XX=xSize*5+XX;
    //System.out.println("Fitness "+XX);
    return XX;
  }
}
  