/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;


/** Shows basic genetic algorithm on a 3D function.
  * <p>
  * Genetic algorithm is used to minimize certain function (defined in class Function3D).
  * All solutions (chromosome) are shown and the best one is in different color.
  * User is allowed to run, stop and step algorithm. Also reset of genetic algorithm
  * (it means new random population) is possible.
  * @see Function3D
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class GA3D extends Applet 
    implements ActionListener, KeyListener, ItemListener, Runnable
{
  /** Pause between generations in run of algorithm */
  final private int STEP_PAUSE=0;
  
  /** Size of population (number of chromosomes), should be even number */
  final private int POP_SIZE=16;
                                 
  /** Thread for running of genetic algorithm in background */
  Thread algThread=null;
                                   
  /** Population of solutions in shown genetic algorithm */
  Population population;

  /** Component for drawing graph and solutions on it */
  Graph3DCanvas Graph=new Graph3DCanvas(new Graph3D());
  
  /** Button Start */
  Button Start=new Button(" Start ");

  /** Button Step */
  Button Step=new Button(" Step ");

  /** Button Stop */
  Button Stop=new Button(" Stop ");

  /** Button Reset */
  Button Reset=new Button(" Reset ");
  
  /** Button Continue */
  Button Continue=new Button(" Continue ");

  Button Change=new Button("Change");

  Checkbox Elitism=new Checkbox("Elitism");
  Checkbox Minimum=new Checkbox("Minimum");
  
  TextField xmin=new TextField("-10",4);
  TextField xmax=new TextField("10",4);
  TextField ymin=new TextField("-10",4);
  TextField ymax=new TextField("10",4);

  TextField popNo=new TextField("0",8);

  Label bestCoo=new Label("Best [X,Y,Z]=[???????????; ???????????; ???????????]");

  ShifterLabel crossover=new ShifterLabel("Crossover 1/100",120,10,10,100);
  ShifterLabel mutation=new ShifterLabel("Mutation 1/1000",200,10,10,200);


  /** crossover probability in percents */
  protected int crossoverProb=80;
  
  /** mutation probability in promiles */
  protected int mutationProb=5;

  TextField expressionText=new TextField("abs(x*x+y*y)",35);
  
  /** Initialization of applet and genetic algorithm */
  public void init() 
  {
    setBGColor();
    Number3DChromosome sample=new Number3DChromosome(32, Graph.g3d);
    sample.randomize();
    sample.countFitness();
    population=new Population(sample, POP_SIZE);
    population.randomize();
    population.popNo=popNo;
    add(Graph);
    
    GridBagLayout gbl=new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.insets=new Insets(2,2,2,2);
    Panel cp=new Panel(gbl);
     
     Panel ep=new Panel();
     //ep.add(new Label("Function",Label.RIGHT));
     ep.add(expressionText);
     expressionText.addKeyListener(this);
     ep.add(Change);
     Change.addActionListener(this);
    
    gbc.fill=GridBagConstraints.CENTER;
    gbc.gridx=0;
    gbc.gridy=2;
    gbl.setConstraints(ep,gbc);
    cp.add(ep);
       
     Panel sp=new Panel(gbl);
     gbc.fill=GridBagConstraints.CENTER;
     gbc.gridx=0;
     gbc.gridy=0;
     Label l=new Label("X min");
     gbl.setConstraints(l, gbc);
     sp.add(l);
     l=new Label("X max");
     gbc.gridx=1;
     gbl.setConstraints(l, gbc);
     sp.add(l);
     l=new Label("Y min");
     gbc.gridx=2;
     gbl.setConstraints(l, gbc);
     sp.add(l);
     l=new Label("Y max");
     gbc.gridx=3;
     gbl.setConstraints(l, gbc);
     sp.add(l);
     gbc.gridx=0;
     gbc.gridy=1;
     gbl.setConstraints(xmin, gbc);
     sp.add(xmin);
     xmin.addKeyListener(this);
     gbc.gridx=1;
     gbl.setConstraints(xmax, gbc);
     sp.add(xmax);
     xmax.addKeyListener(this);
     gbc.gridx=2;
     gbl.setConstraints(ymin, gbc);
     sp.add(ymin);
     ymin.addKeyListener(this);
     gbc.gridx=3;
     gbl.setConstraints(ymax, gbc);
     sp.add(ymax);
     ymax.addKeyListener(this);
     
     gbc.insets=new Insets(2,35,2,2);
     gbc.gridx=4;
     gbc.gridy=0;
     l=new Label("Generation");
     gbl.setConstraints(l, gbc);
     sp.add(l);
     gbc.gridy=1;
     gbl.setConstraints(popNo, gbc);
     sp.add(popNo);
     gbc.insets=new Insets(2,2,2,2);
     

    gbc.gridx=0;
    gbc.gridy=3;
    gbl.setConstraints(sp,gbc);
    cp.add(sp);       
    
     Panel pp=new Panel();
     pp.add(crossover); 
     crossover.setValue(crossoverProb);
     pp.add(mutation);
     mutation.setValue(mutationProb);

    gbc.gridx=0;
    gbc.gridy=4;
    gbl.setConstraints(pp,gbc);
    cp.add(pp);       

     pp=new Panel();
     pp.add(Elitism);
     Elitism.addItemListener(this); 
     Elitism.setState(true);
     pp.add(Minimum);
     Minimum.addItemListener(this); 
     Minimum.setState(true);
     
    gbc.gridx=0;
    gbc.gridy=5;
    gbl.setConstraints(pp,gbc);
    cp.add(pp);       

     Panel bp=new Panel();
     bp.add(Start); 
     Start.addActionListener(this);
     bp.add(Step);
     Step.addActionListener(this);
     bp.add(Stop);
     Stop.addActionListener(this);
     bp.add(Continue);
     Continue.addActionListener(this);
     bp.add(Reset);
     Reset.addActionListener(this);

    gbc.gridx=0;
    gbc.gridy=1;
    gbl.setConstraints(bp,gbc);
    cp.add(bp);       
    
    gbc.gridx=0;
    gbc.gridy=0;
    gbl.setConstraints(bestCoo,gbc);
    cp.add(bestCoo);

    add(cp);    
    Graph.initPaint(); 
    resetGraph();
  }
  
  /** Sets the background color, which is specified in Applet parameter BGCOLOR.
    * For using, just add to applet tag in HTML something like
    * "<"PARAM name=BGCOLOR value="00ff40">".
    */
  protected void setBGColor()
  {
    try 
      {
        Color bg = new Color(Integer.parseInt(getParameter("BGCOLOR"), 16));
        setBackground(bg);
      } catch (NumberFormatException e) {}
        catch (NullPointerException e) {}
  }
  
  /** Handles actions from buttons */
  public void actionPerformed(ActionEvent e)
  {
    String command=e.getActionCommand();
    if(command.equals(Start.getLabel())) actionStart();
    if(command.equals(Stop.getLabel())) actionStop();
    if(command.equals(Reset.getLabel())) actionReset();
    if(command.equals(Step.getLabel())) actionStep();
    if(command.equals(Change.getLabel())) resetGraph();
    if(command.equals(Continue.getLabel())) actionContinue();
  }

  public void keyPressed(KeyEvent e)  
  {
    if(e.getKeyCode()==KeyEvent.VK_ENTER) resetGraph();
  }

  public void keyReleased(KeyEvent e)      
  {
  }
  
  public void keyTyped(KeyEvent e)      
  {
  }
  
  /** Handles actions from checkboxes and item selectors */
  public void itemStateChanged(ItemEvent e) 
  {
    population.elitism=Elitism.getState(); 
    Graph.g3d.lookingForMaximum=!Minimum.getState(); 
  }

  /** Recalculates and redraws the graph. */
  protected void resetGraph()
  {
    double aXMin=-10;
    double aXMax=10;
    double aYMin=-10;
    double aYMax=10;

    Function3D.parseFunction(expressionText.getText());

    try {
         aXMin=(new Double(xmin.getText())).doubleValue();
         aXMax=(new Double(xmax.getText())).doubleValue();
         aYMin=(new Double(ymin.getText())).doubleValue();
         aYMax=(new Double(ymax.getText())).doubleValue();
        } catch(NumberFormatException e) 
           {
             aXMin=-10; aXMax=10;
             aYMin=-10; aYMax=10;
             Function3D.wasError=true;
           }  
    Graph.g3d.setScale(aXMin,aXMax,aYMin,aYMax);
    Graph.resetAll();
  }
  
  /** Button Start starts genetic algorithm (as a thread) */
  protected void actionStart()
  {
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }
  
  /** Button Reset resets genetic algorithm */
  protected void actionReset()
  {
    population.randomize();
    Graph.paint();
  }
  
  /** Button Continue continues running genetic algorithm (as a thread) */
  protected void actionContinue()
  {
    if(algThread!=null) return;
    algThread = new Thread(this);
    algThread.start();
  }

  /** Button Stop stops genetic algorithm (the thread) */
  protected void actionStop()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    Graph.paint();
  }
  
  protected void changes()
  {
    if(crossover.changed()) crossoverProb=crossover.getValue();
    if(mutation.changed()) mutationProb=mutation.getValue();
  }

  /** Button Step performs one step (on generation) of genetic algorithm */
  protected void actionStep()
  {
    changes();
    population.newGeneration(crossoverProb, mutationProb/10.0);
    //Progress.addValue((int)(population.fitnessSum/population.size),1);
    //Progress.addValue(population.pop[0].fitness,0);
    Graph.paint();
    bestCoo.setText(Graph.theBest);
    //Progress.paint();
  }
  
  /** Defines run of genetic algorithm thread */
  public void run()
  {
    Graph.setPopulation(population);
    Graph.paint();
    try { Thread.sleep(STEP_PAUSE);
        } catch (InterruptedException e) {}
    while(true)
      {
         actionStep();   
         try { Thread.sleep(STEP_PAUSE);
             } catch (InterruptedException e) {}
      }
  }

  /** Finishing of the applet and destroying thread. */
  public void destroy()
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.destroy();
  }

  /** Finishing of the applet and destroying thread.
    * @exception Throwable successor can throw an exception
    */
  protected void finalize() throws Throwable
  {
    if(algThread!=null)
      {
        algThread.stop();
        algThread=null;
      }
    super.finalize();
  }

  /** For running of applet without browser or appletviewer (it is then possible
    * to run this applet as a class directly).
    * @see GAApplication
    */
  public static void main(String[] args)
  {
    Applet applet = new GA3D();
    Frame frame = new GAApplication("Genetic Algorithm in 3D", applet, 390, 630);
  }

  /** Returns information about parameters of applet. */
  public String[][] getParameterInfo() 
  {
    String[][] info = 
     {
      {"BGCOLOR", "color (6 hexa digits)", "Background color, for example \"00ff40\"."},
     };
        return info;
  }

  /** Returns information about applet. */
  public String getAppletInfo() 
  {
    return "Title: Genetic algorithm in 3D function \n"+
           "Author: Marek Obitko, obitko@email.cz \n"+
           "Applet shows GA in 3D function and allows changes of the function.";
  }
}