/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;
import java.awt.Canvas;
import java.awt.Dimension; 
import java.awt.Color;
import java.awt.Image;

/** Component for drawing graph and solutions on it.
  * <p> It uses double buffering to prevent blinking.
  * @see Function
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class GraphCanvas extends Canvas
{

  /** Size of component */
  protected Dimension mySize;
  
  /** Population being displayed on the graph. */
  protected Population population=null;
  
  /** Graphics of component */
  protected Graphics gr;
  
  /** Graphics of offline buffer */
  protected Graphics offgr;
  
  /** Image of offline buffer */
  protected Image offscreen;
  
  /** Constructor which makes standart size graph */
  public GraphCanvas()
  {
    this(690,140);
  }
  
  /** Constructor where you can specify size of component */
  public GraphCanvas(int xsize, int ysize)
  {
    super();
    mySize=new Dimension(xsize,ysize);
  }

  /** Draws graph on graphics.
    * @param g graphics which the graph is drawn on
    */
  protected void drawGraph(Graphics g)
  {
    g.setColor(Color.black);  // box around
    g.drawRect(0, 0, mySize.width - 1, mySize.height - 1);
    for(int i=0; i<mySize.height; i+=20)
      {
        g.drawLine(0,i,mySize.width-1,i);
      }                                 
    g.setColor(Color.blue);  // graph itself
    int lasty=0, newy;
    for(int i=0; i<mySize.width-1;i++)
      {
        newy=Function.value(i);
        if(i==0)
          {
            lasty=newy;
          }
          else 
          {
            g.drawLine(i-1,lasty,i,newy);
            lasty=newy;
          }
      }
  }
  
  /** Draws population on graphics.
    * @param g graphics which the solutions are drawn on
    */
  protected void drawPopulation(Graphics g)
  {
    if(population==null) return; 
    int x;
    g.setColor(Color.green);
    for(int i=1; i<population.size; i++)
      {
        x=(int)(690*((NumberChromosome)(population.pop[i])).value()/4294967296L);
		    g.drawLine(x,1,x,mySize.width-2);
      }
    g.setColor(Color.red);  // the best is red
    x=(int)(690*((NumberChromosome)(population.pop[0])).value()/4294967296L);
    g.drawLine(x,1,x,mySize.width-2);
  }
  
  /** Initializes double buffering.
    * This should be called before using method paint.
    * @see #paint
    */
  public void initPaint()
  {
    gr = this.getGraphics();
    offscreen = this.createImage(mySize.width, mySize.height);
    offgr = offscreen.getGraphics();
  }
  
  /** This method acts like repaint(), but it prevents blinking by buffering
    * an image.
    */
  public void paint()
  {
    if(offgr==null) initPaint();
    offgr.setColor(getBackground());
    offgr.fillRect(0, 0, mySize.width, mySize.height);
    offgr.setColor(getForeground());
    paint(offgr);
    gr.drawImage(offscreen, 0, 0, this);
    //paint(getGraphics());
  }
  
  /** Paints graph and solutions on it (if there are any). */
  public void paint(Graphics g) 
  {
     drawGraph(g);  
     drawPopulation(g); 
  }
  
  /** Sets population for drawing.
    * @param aPopulation population, which will be shown on the graph; use null for
    *        drawing no population on graph
    */ 
  public void setPopulation(Population aPopulation)
  {
    population=aPopulation;
  }
  
  public Dimension getPreferredSize()
  {
    return mySize;
  }

  public Dimension getMaximumSize()
  {
    return mySize;
  }

  public Dimension getMinimumSize()
  {
    return mySize;
  }

}