/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Represents one joint in creature.
  * @see WalkGA
  * @see Creature
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Joint 
{
  
  public Joint jlist[]=null;
  public int jcount=0;
  
  public double minAngle;
  public double maxAngle;
  public double angle;
  public double lastAngleVar;
  public double var;
  public int distance;
  
  public int id;
  
  static protected int idCount;
  
  public static void nullIdCount()
  {
    idCount=0;
  }

  static 
  {
    nullIdCount();
  }
  
  public Joint(int MinAngle, int MaxAngle, int Distance)
  {
    minAngle=Math.PI*MinAngle/180;
    maxAngle=Math.PI*MaxAngle/180;
    angle=(maxAngle+minAngle)/2;
    var=Math.abs(maxAngle-minAngle)/2;
    lastAngleVar=0;
    distance=Distance;
    id=idCount;
    idCount++;
  }
  
  public boolean addJoint(int aId, Joint j)
  {
    if(id==aId) 
      {
        addJoint(j);
        return true;
      }  
     else 
      {
        for(int i=0; i<jcount; i++) 
          if(jlist[i].addJoint(aId,j)) return true;
        return false;
      }    
  }

  public void resetVar()
  {
    lastAngleVar=0; 
    for(int i=0; i<jcount; i++) jlist[i].resetVar();
  }

  public void addJoint(Joint j)
  {
    Joint njl[] = new Joint[jcount+1];
    for(int i=0; i<jcount; i++) njl[i]=jlist[i];
    njl[jcount]=j;
    ++jcount;
    jlist=new Joint[jcount];
    for(int i=0; i<jcount; i++) jlist[i]=njl[i];
  }
  
  public void startComputeDraw(Creature c, int x, int y, double alfa)
  {
    //System.out.println("start id"+id+" x="+x+" y="+y+" alfa="+alfa);
    //c.dump();
    c.ftEnd--;
    for(int i=0; i<jcount; i++)
      {
        c.ftEnd++;
        c.from[c.ftEnd]=id;
        jlist[i].computeDraw(c,x,y,alfa);
      }
    //System.out.print("end ---"+id+"-"); c.dump();
  }
  
  public void computeDraw(Creature c, int x, int y, double alfa)
  {
    double variation;
    try { variation=c.chr.v[id][c.position]; } 
       catch(NullPointerException e) { variation=0; System.out.println("!-Null-!"); } 
    lastAngleVar+=variation;
    
    if(lastAngleVar<-1) lastAngleVar=-1;
    if(lastAngleVar>1) lastAngleVar=1;
    
    //lastAngleVar=variation;
    //System.out.println("v["+id+"]["+c.position+"]="+variation);   
    //alfa += ( angle + var*variation );
    alfa += ( angle + var*lastAngleVar );
    if(c.xyEnd<id) c.xyEnd=id;
    int mx = c.x[id]= (int) (x+distance*Math.cos(alfa));
    int my = c.y[id]= (int) (y+distance*Math.sin(alfa));

    //System.out.println("      id"+id+" x="+mx+" y="+my+" alfa="+alfa);
    //c.dump();

    c.to[c.ftEnd]=id;
    for(int i=0; i<jcount; i++)
      {
        c.ftEnd++;
        c.from[c.ftEnd]=id;
        jlist[i].computeDraw(c,mx,my,alfa);
      }
    //System.out.print("end ---"+id+"-"); c.dump();
  }

}
  