/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.lang.Math;

/** Lexical analyzer for expression.
  * @see Expression
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class LexicalAnalyzer
{
  /** Analyzed string. */
  private String string;
  
  private int length, read;
  
  /** Type of symbol analyzed. */
  public int type; 
  
  /** Type of lexical symbol. */
  public static final int EOIType=0, // end of input
                          numberType=1, // number 1, 1.1, .1 (no exponent)
                          XType=2,
                          EType=3,
                          PIType=4,
                          leftType=5,
                          rightType=6,
                          powerType=7,
                          plusType=8,
                          minusType=9,
                          multiplyType=10,
                          divideType=11,
                          factorialType=12,
                          sinType=13,
                          asinType=14,
                          cosType=15,
                          acosType=16,
                          tanType=17,
                          atanType=18,
                          sinhType=19,
                          asinhType=20,
                          coshType=21,
                          acoshType=22,
                          tanhType=23,
                          atanhType=24,
                          logType=25,
                          lnType=26,
                          sqrType=27,
                          sqrtType=28,
                          absType=29,
                          intType=30,
                          YType=31;

  public double value; /* for numberType */

  /** Reads first symbol. */
  public LexicalAnalyzer(String s) throws ParseError 
  {
    string=s; 
    length=s.length(); 
    read=0;
    symbolRead();
  }
  
  /** Reads next symbol. */
  public void next() throws ParseError 
  {
    if(type!=EOIType) symbolRead();
  }
    
  /** Error at this place. */
  public void throwError() throws ParseError 
  {
    throw new ParseError(string,read);
  }
  
  /** Which char will be next.  */
  private int charLook() 
  {
    if(read==length) return -1;
    int c=string.charAt(read);
    if(c>='A'&&c<='Z') return c+'a'-'A';
    return c; 
  }
  
  /** Skip to next char. */
  private void charNext()
  {
    if(read<length) read++; 
  }
  
  /** Is character number? */
  private boolean charIsNumber()
  {
    int c;
    c=charLook();
    return c>='0'&&c<='9'; 
  }
  
  /** Reads one lexical symbol and decides which one it is. */
  private void symbolRead() throws ParseError
  {
    double mask;
    while(true) switch(charLook())
      {
        case -1:   type=EOIType;
                   return;
        case ' ': 
        case '\t': charNext();
                   break;
        case '(':  charNext();
                   type=leftType;
                   return;
        case ')':  charNext();
                   type=rightType;
                   return;
        case '^':  charNext();
                   type=powerType;
                   return;
        case '+':  charNext();
                   type=plusType;
                   return;
        case '-':  charNext();
                   type=minusType;
                   return;
        case '*':  charNext();
                   type=multiplyType;
                   return;
        case '/':  charNext();
                   type=divideType;
                   return;
        case '!':  charNext();
                   type=factorialType;
                   return;
        case 'a': charNext();
                  switch(charLook())
                   {case 's':
                     charNext();
                     switch(charLook())
                      {case 'i':
                        charNext();
                        switch(charLook())
                         {case 'n':
                           charNext();
                           switch(charLook())
                            {case 'h':
                              charNext();
                              type=asinhType;
                              return;
                             default:
                              type=asinType;
                              return;
                            }
                          default: throw new ParseError(string,read);
                         }
                       default: throw new ParseError(string,read);
                      }
                    case 'b':
                     charNext();
                     switch(charLook())
          {
           case 's':
             charNext();
             type=absType;
             return;
           default: throw new ParseError(string,read);
          }
      case 'c':
        charNext();
        switch(charLook())
          {
           case 'o':
             charNext();
             switch(charLook())
         {
          case 's':
            charNext();
            switch(charLook())
              {
               case 'h':
           charNext();
           type=acoshType;
           return;
               default:
           type=acosType;
           return;
              }
          default: throw new ParseError(string,read);
         }
           default: throw new ParseError(string,read);
          }
      case 't':
        charNext();
        switch(charLook())
          {
           case 'a':
             charNext();
             switch(charLook())
         {
          case 'n':
            charNext();
            switch(charLook())
              {
               case 'h':
           charNext();
           type=atanhType;
           return;
               default:
           type=atanType;
           return;
              }
          default: throw new ParseError(string,read);
         }
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 'c': charNext();
         switch(charLook())
           {
      case 'o':
        charNext();
        switch(charLook())
          {
           case 's':
             charNext();
             switch(charLook())
         {
          case 'h':
            charNext();
            type=coshType;
            return;
          default:
            type=cosType;
            return;
         }
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 'i': charNext();
         switch(charLook())
           {
      case 'n':
        charNext();
        switch(charLook())
          {
           case 't':
             charNext();
             type=intType;
             return;
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 'l': charNext();
         switch(charLook())
           {
      case 'n':
        charNext();
        type=lnType;
        return;
      case 'o':
        charNext();
        switch(charLook())
          {
           case 'g':
             charNext();
             type=logType;
             return;
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 'e': charNext();
         type=EType;
         return;
     case 'p': charNext();
         switch(charLook())
           {
      case 'i': charNext();
          type=PIType;
          return;
      default: throw new ParseError(string,read);
           }
     case 's': charNext();
         switch(charLook())
           {
      case 'i':
        charNext();
        switch(charLook())
          {
           case 'n':
             charNext();
             switch(charLook())
         {
          case 'h':
            charNext();
            type=sinhType;
            return;
          default:
            type=sinType;
            return;
         }
           default: throw new ParseError(string,read);
          }
      case 'q':
        charNext();
        switch(charLook())
          {
           case 'r':
             charNext();
             switch(charLook())
         {
          case 't':
            charNext();
            type=sqrtType;
            return;
          default:
            type=sqrType;
            return;
         }
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 't': charNext();
         switch(charLook())
           {
      case 'a':
        charNext();
        switch(charLook())
          {
           case 'n':
             charNext();
             switch(charLook())
         {
          case 'h':
            charNext();
            type=tanhType;
            return;
          default:
            type=tanType;
            return;
         }
           default: throw new ParseError(string,read);
          }
      default: throw new ParseError(string,read);
           }
     case 'x': charNext();
         type=XType;
         return;
     case 'y': charNext();
         type=YType;
         return;
     case '.': type=numberType;
         value=0;
         charNext();
         if(!charIsNumber()) throw new ParseError(string,read);
                     mask=1;
                     while(charIsNumber())
                       { value+=(mask/=10)*(charLook()-'0');
                         charNext(); }
                     return;
           default: if(!charIsNumber()) throw new ParseError(string,read);
                    type=numberType;
                    value=0;
                    while(charIsNumber())
                      { value*=10; value+=charLook()-'0';
                        charNext(); }
                    if(charLook()!='.') return;
                    charNext();
                    if(!charIsNumber()) throw new ParseError(string,read);
                    mask=1;
                    while(charIsNumber())
                      { value+=(mask/=10)*(charLook()-'0');
                        charNext(); }
                    return;
          } /* while and switch */
      } /* symbol read */
  } /* class LexicalAnalyzer */
  
  