/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;
import java.awt.Color;

/** Chromosome representing two numbers in a binary string. Used for representing
  * two coordinates (x and y).
  * <p>
  * Defines methods for evaluating numbers value and fitness function.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class Number3DChromosome extends Chromosome
{

  /** array of bits in chromosome */
  public int xbit[];
  /** array of bits in chromosome */
  public int ybit[];
  
  /** number of bits in chromosome */
  public int length;

  public Graph3D g3d;
  
  /** Creates number chromosome.
    * @param alength length of the string (so the number will be from 0 till 2^alength)
    */
  public Number3DChromosome(int alength, Graph3D aGraph3D)
  {
    length=alength;
    xbit=new int[length];
    ybit=new int[length];
    g3d=aGraph3D;
  }
  
  /** Returns exact copy of chromosome. */
  public Chromosome copy()
  {
    Number3DChromosome result=new Number3DChromosome(length, g3d);
    for(int i=0; i<length; i++) 
      {
        result.xbit[i]=xbit[i];
        result.ybit[i]=ybit[i];
      }
    result.fitness=fitness;
    return result;
  }

  
  /** Returns new chromosomes from crossover.
    * @see StringChromosome#crossover(Chromosome, double)
    */
  public Chromosome[] crossover(Chromosome other, double probability)
  {
    Number3DChromosome[] result=new Number3DChromosome[2];

    //suggestion: try uniform crossover !

    if(probability<(Math.random()*100))
       {
         lastCrossoverPoint=-1;
         result[0]=(Number3DChromosome)((Number3DChromosome)this).copy();
         result[1]=(Number3DChromosome)((Number3DChromosome)other).copy();
       }  
     else
      {
        result[0]=new Number3DChromosome(length,g3d);
        result[1]=new Number3DChromosome(length,g3d);
        int crosspoint=(int)(Math.random()*length);  
        lastCrossoverPoint=crosspoint;
        int i=0;
        for(; i<crosspoint;i++)
          {
            result[0].xbit[i]=xbit[i];
            result[1].xbit[i]=((Number3DChromosome)other).xbit[i];
          }
        for(;i<length;i++)
          {
            result[1].xbit[i]=xbit[i];
            result[0].xbit[i]=((Number3DChromosome)other).xbit[i];
          }  
        i=0;
        for(; i<crosspoint;i++)
          {
            result[0].ybit[i]=ybit[i];
            result[1].ybit[i]=((Number3DChromosome)other).ybit[i];
          }
        for(;i<length;i++)
          {
            result[1].ybit[i]=ybit[i];
            result[0].ybit[i]=((Number3DChromosome)other).ybit[i];
          }  
      }
    return result;
  }

  /** Value of the chromosome x number.
    * @return integer number represented by chromosome binary string
    */
  public long xvalue()
  {
    long val=0;
    long base=1;
     
    for(int i=0;i<length;i++)
      {
        val+=base*xbit[i];
        base*=2;
      }                      
        
    return val;
  }

  /** Value of the chromosome y number.
    * @return integer number represented by chromosome binary string
    */
  public long yvalue()
  {
    long val=0;
    long base=1;
     
    for(int i=0;i<length;i++)
      {
        val+=base*ybit[i];
        base*=2;
      }                      
        
    return val;
  }
  
  /** Prints the contents of chromosome string and the value. */
  public void printSelf()
  {
    System.out.print("Number(3D): ");
    for(int i=0; i<length; i++) System.out.print(xbit[i]);
    System.out.print(", ");
    for(int i=0; i<length; i++) System.out.print(ybit[i]);
    System.out.print(" = "+"["+xvalue()+","+yvalue()+"]");
    System.out.println();
  }

  /** Counts fitness of the chromosome.
    * Uses class Function for evaluating.
    * @see Function
    */
  public void countFitness()
  {
    fitness=g3d.countFitness(xvalue(),yvalue());
    //System.out.println("z n3d chromosome (x,y): ("+xvalue()+","+yvalue()+"): "+fitness);
  }

  /** Performs mutation on the chromosome.
    * @param probability specifies probability of mutation on each bit in chromosome string 
    */
  public void mute(double probability)
  {
    for(int i=0; i<length; i++)
      {
        if((Math.random()*100)<probability)
          if(xbit[i]==1) xbit[i]=0; else xbit[i]=1;
        if((Math.random()*100)<probability)
          if(ybit[i]==1) ybit[i]=0; else ybit[i]=1;
      }   
  }   

  /** Randomizes the chromosome binary string.
    */
  public void randomize()
  {
    for(int i=0; i<length; i++)
      {
        if(Math.random()<0.5) xbit[i]=0; else xbit[i]=1;
        if(Math.random()<0.5) ybit[i]=0; else ybit[i]=1;
      }
  }
  
}