/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;
import java.awt.Color;

/** Chromosome representing number in a binary string.
  * <p>
  * Defines methods for evaluating number value and fitness function.
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class NumberChromosome extends StringChromosome
{

  /** Creates number chromosome.
    * @param alength length of the string (so the number will be from 0 till 2^alength)
    */
  public NumberChromosome(int alength)
  {
    super(alength);
  }
  
  /** Returns exact copy of chromosome. */
  public Chromosome copy()
  {
    NumberChromosome result=new NumberChromosome(length);
    for(int i=0; i<length; i++) result.bit[i]=bit[i];
    result.fitness=fitness;
    return result;
  }

  
  /** Returns new chromosomes from crossover.
    * @see StringChromosome#crossover(Chromosome, int)
    */
  public Chromosome[] crossover(Chromosome other, double probability)
  {
    NumberChromosome[] result=new NumberChromosome[2];
    
    // System.out.println("Crossover");

    if(probability<(Math.random()*100))
       {
         lastCrossoverPoint=-1;
         result[0]=(NumberChromosome)((NumberChromosome)this).copy();
         result[1]=(NumberChromosome)((NumberChromosome)other).copy();
       }  
     else
      {
        result[0]=new NumberChromosome(length);
        result[1]=new NumberChromosome(length);
        int crosspoint=(int)(Math.random()*length);  
        lastCrossoverPoint=crosspoint;
        int i=0;
        for(; i<crosspoint;i++)
          {
            result[0].bit[i]=bit[i];
            result[1].bit[i]=((NumberChromosome)other).bit[i];
          }
        for(;i<length;i++)
          {
            result[1].bit[i]=bit[i];
            result[0].bit[i]=((NumberChromosome)other).bit[i];
          }  
      }
    return result;
  }

  /** Value of the chromosome.
    * @return integer number represented by chromosome binary string
    */
  public long value()
  {
    long val=0;
    long base=1;
     
    for(int i=0;i<length;i++)
      {
        val+=base*bit[i];
        base*=2;
      }                      
        
    return val;
  }
  
  /** Paints the chromosome shape without any information 
    * to graphics g on coordinates x and y. It can be used for
    * visualization of empty chromosome shape. 
    */
  public void paintEmpty(Graphics g, int x, int y)
  {
    g.setColor(Color.black);
    g.drawRect(x,y,8,7*length+1);
  }
    
  /** Paints the chromosome to graphics g on coordinates x and y. It can be used for
    * visualization of chromosome. 
    */
  public void paint(Graphics g, int x, int y)
  {
    g.setColor(Color.black);
    g.drawRect(x,y,8,7*length+1);
    for(int i=0; i<length; i++)
      {
        if(bit[i]==1) g.setColor(Color.red);
                 else g.setColor(Color.blue);
        g.fillRect(x+2,y+i*7+2,5,5);
      }  
    g.setColor(Color.black);
  }

  /** Prints the contents of chromosome string and the value. */
  public void printSelf()
  {
    System.out.print("Number: ");
	  for(int i=0; i<length; i++) System.out.print(bit[i]);
	  System.out.print(" = "+value());
	  System.out.println();
  }

  /** Counts fitness of the chromosome.
    * Uses class Function for evaluating.
    * @see Function
    */
  public void countFitness()
  {
    //fitness=141-Function.value((long)(690*value()/4294967296L));
    fitness=Function.value((long)(690*value()/4294967296L));
  }

}