/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

/** Chromosome with a permutation .
  * <p>
  * Defines all the operators of genetic algorithm except evaluating of fitness function.
  * You should override method countFitness() in your derived classes.
  * Crossover is defined as standart one point crossover.
  * @see #countFitness
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class PermutationChromosome extends Chromosome
{
  /** array of numbers (permutation) in chromosome */
  public int perm[];
  
  /** number of numbers in chromosome */
  public int length;

  /** Creates new permutation chromosome.
    * @param alength length of the permutation
    */
  public PermutationChromosome(int alength)
  {
    length=alength;
    perm=new int[length];
  }
  
  /** This method evaluates fitness as zero in all cases, so you should override it
    * in derived classes. */
  public void countFitness()
  {
    fitness=0;
  }
  
  /** Prints the contents of permutation chromosome. */
  public void printSelf()
  {
    System.out.print("Permutation: ");
	  for(int i=0; i<length; i++) System.out.print(perm[i]+",");
	  System.out.println();
  }

  /** Returns exact copy of chromosome. */
  public Chromosome copy()
  {
    PermutationChromosome result=new PermutationChromosome(length);
    for(int i=0; i<length; i++) result.perm[i]=perm[i];
    result.fitness=fitness;
    return result;
  }

  /** Returns null - override in derived clases.
    * @return array of two new chromosomes - children
    */
  public Chromosome[] crossover(Chromosome other, double probability)
  {
    return null;
  }
  
  /** Performs mutation on the chromosome.
    * @param probability specifies probability of switching order at each position
    */
  public void mute(double probability)
  {
    for(int i=0; i<length; i++)
     if((Math.random()*100)<probability)
      {
        int p1=(int)(Math.random()*length);
        int p2=(int)(Math.random()*length);
        int tmp=perm[p1];
        perm[p1]=perm[p2];
        perm[p2]=tmp;
      }
  }   

  /** Randomizes the chromosome permutation.
    */
  public void randomize()
  {
    for(int i=0; i<length; i++)
      perm[i]=i;
    for(int i=0; i<length/2; i++)
      {
        int p1=(int)(Math.random()*length);
        int p2=(int)(Math.random()*length);
        int tmp=perm[p1];
        perm[p1]=perm[p2];
        perm[p2]=tmp;
      }
  }

}