/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.*;

/** Component for displaying shifter, label and value of the shifter together.
  * Label with the number are dispayed up, shifter is down. It creates one linked
  * component with features like shifter, so you can use the same methods as in the 
  * shifter for managing the value.
  * @see Shifter
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
class ShifterLabel extends Panel
{

  Shifter shifter;
  Label label;
  Label number;
   
  /** Creates new shifter with label with the specified name, standart size 
    * and range 0-100. */
  public ShifterLabel(String name)
  {
    this(name, 100);
  }

  /** Creates new shifter with label with the specified name and width. 
    * Range is 0-100. */
  public ShifterLabel(String name, int width)
  {
    this(name, width, 10, 10, 100);
  }
  
  /** Creates new shifter with label with the specified name and the shifter with
    * all specified parameters. 
    * @see Shifter#Shifter(int, int, int, double) 
    */
  public ShifterLabel(String name, int width, int height, int inBoxWidth, double range)
  {
    shifter=new Shifter(width, height, inBoxWidth, range);
    setLayout(new BorderLayout());
    label=new Label(name);
    number=new Label(String.valueOf((int)range-1));
    this.add("West",label);
    this.add("East",number);
    this.add("South",shifter);
    shifter.setValueLabel(number);
  }
  
  /** Returns value of the shifter.
    * @return integer value in the range from 0 to Range
    */
  public int value() 
  {
    return shifter.getValue();
  }

  /** Returns value of the shifter.
    * @return integer value in the range from 0 to Range
    */
  public int getValue() 
  {
    return shifter.getValue();
  }

  /** Sets new value for the shifter. */
  public void setValue(int value)
  {
    shifter.setValue(value);
  }
  
  /** Answers if the value of shifter has been changed from the time of last asking. */
  public boolean changed()
  {
    return shifter.changed();
  }
  
}