/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/


/** Chromosome with a string of 0 and 1 (binary string).
  * <p>
  * Defines all the operators of genetic algorithm except evaluating of fitness function.
  * You should override method countFitness() in your derived classes.
  * Crossover is defined as standart one point crossover.
  * @see #countFitness
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class StringChromosome extends Chromosome
{
  /** array of bits in chromosome */
  public int bit[];
  
  /** number of bits in chromosome */
  public int length;

  /** Creates new binary string chromosome.
    * @param alength length of the string
    */
  public StringChromosome(int alength)
  {
    length=alength;
    bit=new int[length];
  }
  
  /** This method evaluates fitness as zero in all cases, so you should override it
    * in derived classes. */
  public void countFitness()
  {
    fitness=0;
  }
  
  /** Prints the contents of chromosome string. */
  public void printSelf()
  {
    System.out.print("String: ");
	  for(int i=0; i<length; i++) System.out.print(bit[i]);
	  System.out.println();
  }

  /** Returns exact copy of chromosome. */
  public Chromosome copy()
  {
    StringChromosome result=new StringChromosome(length);
    for(int i=0; i<length; i++) result.bit[i]=bit[i];
    result.fitness=fitness;
    return result;
  }

  /** Returns new chromosomes from crossover. Performs one point crossover.
    * Crossover is made of this (called chromosome) and other (parameter).
    * The probability of crosover specifies probability of performing crossover. If the 
    * crossover is not performed, the exact copies of parents are returned. Otherwise
    * one crossover point is randomly chosen and two new childrens are returned.
    * @return array of two new chromosomes - children
    */
  public Chromosome[] crossover(Chromosome other, double probability)
  {
    StringChromosome[] result=new StringChromosome[2];
    
    if(probability<(Math.random()*100))  
       { // no crossove 
         lastCrossoverPoint=-1;
         result[0]=(StringChromosome)((StringChromosome)this).copy();
         result[1]=(StringChromosome)((StringChromosome)other).copy();
       }  
     else
      { // crossover
        result[0]=new StringChromosome(length);
        result[1]=new StringChromosome(length);
        int crosspoint=(int)(Math.random()*length);  // crossover point
        lastCrossoverPoint=crosspoint;
        int i=0;
        for(; i<crosspoint;i++)
          {
            result[0].bit[i]=bit[i];  // copy to crossover point
            result[1].bit[i]=((StringChromosome)other).bit[i];
          }
        for(;i<length;i++)
          {
            result[1].bit[i]=bit[i];  // switched copy till end
            result[0].bit[i]=((StringChromosome)other).bit[i];
          }  
      }
    return result;
  }
  
  /** Performs mutation on the chromosome.
    * @param probability specifies probability of mutation on each bit in chromosome string 
    */
  public void mute(double probability)
  {
    for(int i=0; i<length; i++)
     if((Math.random()*100)<probability)
       if(bit[i]==1) bit[i]=0; else bit[i]=1;
  }   

  /** Randomizes the chromosome binary string.
    */
  public void randomize()
  {
    for(int i=0; i<length; i++)
      if(Math.random()<0.5) bit[i]=0; else bit[i]=1;
  }

}