/*
======================================================
====      Java Classes For Genetic Algorithms     ====
==== (c) Marek Obitko, 1998-1999, obitko@email.cz ====
======================================================

This file is a part of Java library for genetic algorithms with 
visualization. You may use this library or its part free of charge
provided you acknowledge the author (Marek Obitko), and the source 
(Learning Environment for Genetic Algorithms, one of the URLs 
below) in your work where you use it (contact the author, if you 
are in doubt). Moreover I would appreciate if you inform me 
(via email) about your work where you used it 
(this is not compulsory).

THIS LIBRARY IS PROVIDED "AS IS", WITHOUT ANY WARRANTY.

This package was designed for learning environment for GAs, see
http://cs.felk.cvut.cz/~xobitko/ga  or

Author is studying Czech Technical University in Prague, 
Faculty of Electrical Engineering, Department of Computer Engineering.
Major part of this library was written at University of Applied Sciency
in Dresden, Germany.
*/

import java.awt.Graphics;
import java.awt.Canvas;
import java.awt.Dimension; 
import java.awt.Color;
import java.awt.Image;

/** Component for drawing walk (movements).
  * <p> It uses double buffering to prevent blinking.
  * @see WalkGA
  * <p>
  * @author Marek Obitko, <A HREF="mailto:obitko@email.cz">obitko@email.cz</A>
  * @version 1.0
  */
public class WalkCanvas extends Canvas
{

  /** Size of component */
  protected Dimension mySize;
  
  /** Graphics of component */
  protected Graphics gr;
  
  /** Graphics of offline buffer */
  protected Graphics offgr;
  
  /** Image of offline buffer */
  protected Image offscreen;
  
  protected Creature creature;
  
  /** Constructor which makes standart size graph */
  public WalkCanvas()
  {
    this(500,300);
  }
  
  /** Constructor where you can specify size of component */
  public WalkCanvas(int xsize, int ysize)
  {
    super();
    mySize=new Dimension(xsize,ysize);
  }

  /** Draws graph on graphics.
    * @param g graphics which the graph is drawn on
    */
  protected void drawGraph(Graphics g)
  {
    g.setColor(Color.black);  // box around
    g.drawRect(0, 0, mySize.width - 1, mySize.height - 1);
    
  }
  
  /** Initializes double buffering.
    * This should be called before using method paint.
    * @see #paint
    */
  public void initPaint()
  {
    gr = this.getGraphics();
    offscreen = this.createImage(mySize.width, mySize.height);
    offgr = offscreen.getGraphics();
  }
  
  /** This method acts like repaint(), but it prevents blinking by buffering
    * an image.
    */
  public void paint()
  {
    if(offgr==null) initPaint();
    offgr.setColor(getBackground());
    offgr.fillRect(0, 0, mySize.width, mySize.height);
    offgr.setColor(getForeground());
    paint(offgr);
    gr.drawImage(offscreen, 0, 0, this);
  }
  
  /** Paints graph and solutions on it (if there are any). */
  public void paint(Graphics g) 
  {
     drawGraph(g);  
     drawCreature(g); 
  }
  
  protected void drawCreature(Graphics g)
  {
    if(creature==null) return; 
    Creature c=creature;
    
    g.setColor(Color.cyan);
    for(int i=0; i<c.xyEnd; i++)
      {
        g.setColor(Color.cyan);
        g.fillRect(c.x[i]-3,c.y[i]-3,7,7);
        g.setColor(Color.black);
        g.drawString(""+i, c.x[i],c.y[i]);
      }
    g.setColor(Color.blue);
    for(int i=0; i<c.ftEnd; i++)
      {
        g.drawLine(c.x[c.from[i]],c.y[c.from[i]],
                   c.x[c. to [i]],c.y[c. to [i]]);
      }
  } 

  public void setCreature(Creature aCreature)
  {
    creature=aCreature;
    creature.ySize=mySize.height-10;
    creature.xSize=mySize.width-2;
    creature.computeDraw();
    paint();
  }

  public void setPosition(int position, int chromosomeNumber)
  {
    creature.computeDraw(position, chromosomeNumber);
    paint();
  }
  
  public Dimension getPreferredSize()
  {
    return mySize;
  }

  public Dimension getMaximumSize()
  {
    return mySize;
  }

  public Dimension getMinimumSize()
  {
    return mySize;
  }

}